<?php

declare(strict_types=1);

namespace Drupal\Tests\symfony_mailer_queue\Kernel;

use Drupal\Core\Queue\DelayableQueueInterface;
use Drupal\Core\Queue\QueueGarbageCollectionInterface;
use Drupal\Core\Queue\QueueInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\language\ConfigurableLanguageManagerInterface;
use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\symfony_mailer\EmailFactoryInterface;
use Drupal\symfony_mailer_queue\Plugin\QueueWorker\SymfonyMailerQueueWorker;
use Drupal\symfony_mailer_queue\QueueableEmailInterface;
use Drupal\symfony_mailer_queue\StaticLanguageNegotiatorInterface;
use Drupal\symfony_mailer_test\MailerTestTrait;
use Drupal\Tests\symfony_mailer_queue\Traits\DelayableQueueRunnerTrait;
use Drupal\Tests\symfony_mailer_queue\Traits\TranslationTestTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Tests that queued emails use correct language for translation.
 *
 * @group symfony_mailer_queue
 */
class SymfonyMailerQueueTranslationTest extends KernelTestBase {

  use DelayableQueueRunnerTrait;
  use MailerTestTrait;
  use TranslationTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'filter',
    'language',
    'locale',
    'symfony_mailer',
    'symfony_mailer_queue',
    'symfony_mailer_test',
    'symfony_mailer_queue_test',
    'system',
    'user',
  ];

  /**
   * The email factory.
   */
  protected EmailFactoryInterface $emailFactory;

  /**
   * The language manager.
   */
  protected ConfigurableLanguageManagerInterface $languageManager;

  /**
   * The static language negotiator.
   */
  protected StaticLanguageNegotiatorInterface $languageNegotiator;

  /**
   * The Symfony mailer queue.
   */
  protected QueueInterface $queue;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {

    parent::setUp();

    ConfigurableLanguage::createFromLangcode('de')->save();
    $this->installSchema('locale', [
      'locales_location',
      'locales_source',
      'locales_target',
    ]);
    $this->addTranslation(
      'This email might change everything',
      // cspell:ignore alles diese e-mail könnte verändern
      'Diese E-Mail könnte alles verändern',
      'de'
    );
    $language_manager = $this->container->get('language_manager');
    $this->assertInstanceOf(ConfigurableLanguageManagerInterface::class, $language_manager);
    $this->languageManager = $language_manager;
    // @phpstan-ignore-next-line Service defined in service provider.
    $language_negotiator = $this->container->get('symfony_mailer_queue.static_language_negotiator', ContainerInterface::NULL_ON_INVALID_REFERENCE);
    $this->assertInstanceOf(StaticLanguageNegotiatorInterface::class, $language_negotiator);
    $this->languageNegotiator = $language_negotiator;
    $this->languageManager->setNegotiator($this->languageNegotiator);

    $this->installConfig([
      'symfony_mailer',
      'symfony_mailer_test',
      'symfony_mailer_queue',
      'symfony_mailer_queue_test',
    ]);

    $this->installEntitySchema('user');

    $this->emailFactory = $this->container->get('email_factory');
    $this->config('system.site')
      ->set('name', 'Example')
      ->set('mail', 'site@example.com')
      ->save();

    /** @var \Drupal\Core\Queue\QueueFactory $queue_factory */
    $queue_factory = $this->container->get('queue');
    $this->queue = $queue_factory->get(SymfonyMailerQueueWorker::QUEUE_NAME);
    $this->queue->createQueue();
    $this->assertInstanceOf(DelayableQueueInterface::class, $this->queue);
    $this->assertInstanceOf(QueueGarbageCollectionInterface::class, $this->queue);
  }

  /**
   * Tests that emails are translated after processing in the queue.
   */
  public function testEmailTranslation(): void {

    $email = $this->emailFactory->newTypedEmail('symfony_mailer_queue', 'translation', 'test@example.com');
    $this->assertInstanceOf(QueueableEmailInterface::class, $email);

    $email->send();
    $this->assertEquals(1, $this->queue->numberOfItems());
    $this->runQueue(SymfonyMailerQueueWorker::QUEUE_NAME);
    $this->assertEquals(0, $this->queue->numberOfItems());

    $this->readMail();
    $this->assertNoError();

    $this->assertSubject('This email might change everything');
    $this->assertBodyContains('Or maybe it changes nothing. Only one way to find out.');

    $language = $this->languageManager->getLanguage('de');
    $this->languageNegotiator->setLanguage($language);
    $this->languageManager->reset();
    $this->languageManager->setConfigOverrideLanguage($language);

    // Emails are sent within a specific language context. Since the queue may
    // be processed in a different environment, we explicitly simulate execution
    // in the default language.
    $email->send();
    $this->assertEquals(1, $this->queue->numberOfItems());
    $this->languageNegotiator->setLanguage();
    $this->languageManager->reset();
    $this->languageManager->setConfigOverrideLanguage();
    $this->runQueue(SymfonyMailerQueueWorker::QUEUE_NAME);
    $this->assertEquals(0, $this->queue->numberOfItems());

    $this->readMail();
    $this->assertNoError();

    // The subject is translated via interface translation. This asserts that
    // the site language context is restored when processing the queued item.
    $this->assertSubject('Diese E-Mail könnte alles verändern');

    // The body text is provided through configuration translation. This asserts
    // that language-specific configuration overrides are properly applied.
    // cspell:ignore finde heraus nichts selbst vielleicht ändert
    $this->assertBodyContains('Oder vielleicht ändert sie gar nichts. Finde es selbst heraus.');
  }

}
